import os
from pathlib import Path

from ament_index_python.packages import get_package_share_directory

from launch.substitutions import Command

from launch.actions import IncludeLaunchDescription, TimerAction
from launch import LaunchDescription
from launch.actions import IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.substitutions import LaunchConfiguration
from launch.actions import DeclareLaunchArgument


from launch_ros.actions import Node
from launch.actions import RegisterEventHandler
from launch.event_handlers import OnProcessExit

def generate_launch_description():

    # Include the robot_state_publisher launch file, provided by our own package. Force sim time to be enabled
    # !!! MAKE SURE YOU SET THE PACKAGE NAME CORRECTLY !!!

    package_name='dcaiti_control' #<--- CHANGE ME

    use_ros2_control = LaunchConfiguration('use_ros2_control')
    track = LaunchConfiguration('track')

    rsp = IncludeLaunchDescription(
                PythonLaunchDescriptionSource([os.path.join(
                    get_package_share_directory(package_name),'launch','rsp.launch.py'
                )]), launch_arguments={'use_sim_time': 'true', 'use_ros2_control': use_ros2_control}.items()
    )
    base_path = Path(get_package_share_directory(package_name))
    world_path = base_path / 'worlds'
    description_path = base_path  /'description'
    
    # Include the Gazebo launch file, provided by the gazebo_ros package

    gazebo = IncludeLaunchDescription(
                PythonLaunchDescriptionSource([os.path.join(
                    get_package_share_directory('ros_gz_sim'), 'launch', 'gz_sim.launch.py')]),
                    launch_arguments=[('gz_args', [f" -r -v 0 {world_path}/generated_worlds/AU2_skidpad.sdf"])],
             )

    # Run the spawner node from the gazebo_ros package. The entity name doesn't really matter if you only have a single robot.
    spawn_entity = Node(package='ros_gz_sim', executable='create',
        arguments=[
            '-topic', 'robot_description',
            '-name', 'spawn_robot',
            '-z', '0.5',
            '-x', '-7',
            ],
        output='screen'
    )

    # Bridge
    bridge = Node(
        package='ros_gz_bridge',
        executable='parameter_bridge',
        arguments=[
            '/clock@rosgraph_msgs/msg/Clock[ignition.msgs.Clock',
            '/boxes@vision_msgs/msg/Detection2DArray@ignition.msgs.AnnotatedAxisAligned2DBox_V',
            '/lidar@sensor_msgs/msg/LaserScan@ignition.msgs.LaserScan',
            '/lidar/points@sensor_msgs/msg/PointCloud2@ignition.msgs.PointCloudPacked',
            'lidar/imu@sensor_msgs/msg/Imu@gz.msgs.IMU'
        ],
        output='screen'
    )
    
    diff_drive_spawner = Node(
        package="controller_manager",
        executable="spawner",
        arguments=["ackermann_steering_controller"],
    )

    joint_broad_spawner = Node(
        package="controller_manager",
        executable="spawner",
        arguments=["joint_broad"],
    )

    delayed_joint_broad_spawner = RegisterEventHandler(
      event_handler=OnProcessExit(
        target_action=spawn_entity,
        on_exit=[joint_broad_spawner],
      )
    )

    delayed_diff_drive_spawner = RegisterEventHandler(
      event_handler=OnProcessExit(
        target_action=spawn_entity,
        on_exit=[diff_drive_spawner],
      )
    )
    
    imu_covariance_adder = Node(
        package='dcaiti_control',
        executable='imu_covariance_adder.py',
        name='imu_covariance_adder',
        output='screen',
        parameters=[
            {'orientation_covariance': [1e-3]*9},
            {'linear_acceleration_covariance': [1e-3]*9},
            {'angular_velocity_covariance': [1e-3]*9},
            {'subscribe_topic': '/lidar/imu'},
            {'publish_topic': '/lidar/imu_covariance'}
        ]
    )

    #ukf_node = Node(
    #        package='robot_localization', 
    #        executable='ukf_node', 
    #        name='ukf_filter_node',
	#        output='screen',
    #        parameters=[str(base_path / 'config' / 'ukf.yml')],
    #        remappings=[('odometry/filtered', 'odometry/local')]           
    #       )


    # Launch them all!
    return LaunchDescription([
        DeclareLaunchArgument(
            'use_ros2_control',
            default_value='true',
            description='Use ros2_control if true'),
        DeclareLaunchArgument(
            'track',
            default_value='AU2_skidpad.lyt',
            description='Which track to load'),
        bridge,
        rsp,
        gazebo,
        spawn_entity,
        delayed_diff_drive_spawner,
        delayed_joint_broad_spawner,
        imu_covariance_adder,
        #ukf_node
    ]) 
